Crate hypotaxis

Crate hypotaxis 

The ChainLink trait with a blanket implementation gives access to methods that can help to create longer method chains.

use hypotaxis::ChainLink;

Traits

ChainLink